@@ -10,10 +10,10 @@ def RenderModel(model, text_height=5, deformed_shape=False, deformed_scale=30,
                 screenshot=None):
   warnings.warn('`RenderModel` will be replaced with `render_model` in a future version of PyNite.', FutureWarning)
   render_model(model, text_height, deformed_shape, deformed_scale, render_loads, color_map,
-               combo_name, case, labels, screenshot)
+               True, combo_name, case, labels, screenshot)
 
 def render_model(model, text_height=5, deformed_shape=False, deformed_scale=30,
-                 render_loads=True, color_map=None, combo_name='Combo 1', case=None, labels=True,
+                 render_loads=True, color_map=None, scalar_bar=True, combo_name='Combo 1', case=None, labels=True,
                  screenshot=None):
     '''
     Renders a finite element model using VTK.
@@ -51,7 +51,8 @@ def render_model(model, text_height=5, deformed_shape=False, deformed_scale=30,
       speed when rendering and interacting with a large model. This can be very useful on plate
       models with large meshes.
     screenshot : string, optional
-      Sends a screenshot to the specified filepath unless set to None. Default is None.
+      Sends a screenshot to the specified filepath unless set to None. The screenshot will be taken
+      when the user closes out of the render window. Default is None.
       
     Raises
     ------
@@ -187,8 +188,9 @@ def render_model(model, text_height=5, deformed_shape=False, deformed_scale=30,
     if (combo_name != None or case != None) and render_loads != False:
         _RenderLoads(model, renderer, text_height, combo_name, case)
     
-    # Render the plates and quads
-    _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, combo_name)
+    # Render the plates and quads, if present
+    if model.Quads or model.Plates:
+      _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar, combo_name)
 
     # Set the window's background to gray
     renderer.SetBackground(0/255, 0/255, 128/255)
@@ -199,7 +201,15 @@ def render_model(model, text_height=5, deformed_shape=False, deformed_scale=30,
     # Render the window
     window.Render()
 
-    # Create a screenshot if requested
+    # Start the interactor. Code execution will pause here until the user closes the window.
+    interactor.Start()
+
+    # Finalize the render window once the user closes out of it. I don't understand everything
+    # this does, but I've found screenshots will cause the program to crash if this line is
+    # omitted.
+    window.Finalize()
+
+    # Create a screenshot of the last view in the window (if requested)
     if screenshot != None:
 
       # Screenshot code
@@ -207,20 +217,18 @@ def render_model(model, text_height=5, deformed_shape=False, deformed_scale=30,
       w2if.SetInput(window)
       w2if.SetInputBufferTypeToRGB()
       w2if.ReadFrontBufferOff()
-      w2if.Update()
+
+      # These next two lines are in the examples and documentation for VTK, but don't seem to do
+      # anything. I've left them here in case I find a bug somewhere down the line that needs
+      # fixing.
+      # w2if.Update()
+      # w2if.Modified()
 
       writer = vtk.vtkPNGWriter()
       writer.SetFileName(screenshot)
       writer.SetInputConnection(w2if.GetOutputPort())
       writer.Write()
 
-      # Close the window
-      window.Finalize()
-      
-    else:
-
-      interactor.Start()
-    
     # Return the window
     return window
 
@@ -1252,6 +1260,18 @@ def _RenderLoads(model, renderer, text_height, combo_name, case):
           ptLoad = VisMoment(position, dir_cos[1, :]*sign, abs(load_value)/max_moment*2.5*text_height, '{:.3g}'.format(load_value), text_height)
         elif load[0] == 'Mz':
           ptLoad = VisMoment(position, dir_cos[2, :]*sign, abs(load_value)/max_moment*2.5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'FX':
+          ptLoad = VisPtLoad(position, [1, 0, 0], load_value/max_pt_load*5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'FY':
+          ptLoad = VisPtLoad(position, [0, 1, 0], load_value/max_pt_load*5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'FZ':
+          ptLoad = VisPtLoad(position, [0, 0, 1], load_value/max_pt_load*5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'MX':
+          ptLoad = VisMoment(position, [1*sign, 0, 0], abs(load_value)/max_moment*2.5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'MY':
+          ptLoad = VisMoment(position, [0, 1*sign, 0], abs(load_value)/max_moment*2.5*text_height, '{:.3g}'.format(load_value), text_height)
+        elif load[0] == 'MZ':
+          ptLoad = VisMoment(position, [0, 0, 1*sign], abs(load_value)/max_moment*2.5*text_height, '{:.3g}'.format(load_value), text_height)
     
         polydata.AddInputData(ptLoad.polydata.GetOutput())
         renderer.AddActor(ptLoad.lblActor)
@@ -1282,6 +1302,12 @@ def _RenderLoads(model, renderer, text_height, combo_name, case):
           distLoad = VisDistLoad(position1, position2, dir_cos[1, :], w1/max_dist_load*5*text_height, w2/max_dist_load*5*text_height, '{:.3g}'.format(w1), '{:.3g}'.format(w2), text_height)
         elif load[0] == 'Fz':
           distLoad = VisDistLoad(position1, position2, dir_cos[2, :], w1/max_dist_load*5*text_height, w2/max_dist_load*5*text_height, '{:.3g}'.format(w1), '{:.3g}'.format(w2), text_height)
+        elif load[0] == 'FX':
+          distLoad = VisDistLoad(position1, position2, [1, 0, 0], w1/max_dist_load*5*text_height, w2/max_dist_load*5*text_height, '{:.3g}'.format(w1), '{:.3g}'.format(w2), text_height)
+        elif load[0] == 'FY':
+          distLoad = VisDistLoad(position1, position2, [0, 1, 0], w1/max_dist_load*5*text_height, w2/max_dist_load*5*text_height, '{:.3g}'.format(w1), '{:.3g}'.format(w2), text_height)
+        elif load[0] == 'FZ':
+          distLoad = VisDistLoad(position1, position2, [0, 0, 1], w1/max_dist_load*5*text_height, w2/max_dist_load*5*text_height, '{:.3g}'.format(w1), '{:.3g}'.format(w2), text_height)
        
         polydata.AddInputData(distLoad.polydata.GetOutput())
         renderer.AddActor(distLoad.lblActors[0])
@@ -1374,7 +1400,7 @@ def _RenderLoads(model, renderer, text_height, combo_name, case):
   polygon_actor.SetMapper(polygon_mapper)
   renderer.AddActor(polygon_actor)
 
-def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, combo_name):
+def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar, combo_name):
   
   # Create a new `vtkCellArray` object to store the elements
   plates = vtk.vtkCellArray()
@@ -1495,10 +1521,21 @@ def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map,
     plate_mapper.SetScalarModeToUsePointData()
     lut.Build()
 
-    # Add the scalar bar for the contours
-    scalar_bar = vtk.vtkScalarBarActor()
-    scalar_bar.SetLookupTable(lut)
-    renderer.AddActor(scalar_bar)
+    # Add the scalar bar for the contours.
+    # Note: After searching online for how to change the font size for the scalar bar label text,
+    # I found that the text automatically sizes itself to the size of the scalar bar. VTK provides
+    # no other controls over the text size. The `vtkTextProperty` commented out below is normally
+    # how text size is controlled in VTK. All the lines of code excecute without an exception, but
+    # the text size is unaffected. The `SetMaximumWidthInPixels` function provides some control
+    # over the text size until the window gets too small.
+    if scalar_bar:
+      scalar = vtk.vtkScalarBarActor()
+      # scalar_text = vtk.vtkTextProperty()
+      # scalar_text.SetFontSize(12)
+      # scalar_bar.SetLabelTextProperty(scalar_text)
+      scalar.SetLookupTable(lut)
+      scalar.SetMaximumWidthInPixels(100)
+      renderer.AddActor(scalar)
     
   # Add the actor for the plates
   renderer.AddActor(plate_actor)
@@ -1537,7 +1574,8 @@ def _MaxLoads(model, combo_name=None, case=None):
         # Find and store the largest point load and moment in the load combination
         if load[3] in model.LoadCombos[combo_name].factors:
 
-          if load[0] == 'Fx' or load[0] == 'Fy' or load[0] == 'Fz':
+          if (load[0] == 'Fx' or load[0] == 'Fy' or load[0] == 'Fz'
+          or load[0] == 'FX' or load[0] == 'FY' or load[0] == 'FZ'):
             if abs(load[1]*model.LoadCombos[combo_name].factors[load[3]]) > max_pt_load:
               max_pt_load = abs(load[1]*model.LoadCombos[combo_name].factors[load[3]])
           else:
